package cz.muni.fi.pb162.interfaces;

import cz.muni.fi.pb162.interfaces.devices.Camera;
import cz.muni.fi.pb162.interfaces.devices.Chargeable;
import cz.muni.fi.pb162.interfaces.devices.Device;
import cz.muni.fi.pb162.interfaces.devices.Smartphone;
import cz.muni.fi.pb162.interfaces.utils.Informer;

/**
 * V tomto deme si ukážeme použitie rozhraní
 * Ukážky rôznych rozhraní nájdete v balíčku devices
 * Pomocná trieda informer je v balíčku utils
 *
 * Rozhranie je v jednoduchosti zoznam metód, ktoré budú obsahovať triedy, ktoré ho budú implementovať
 * V rozhraní sa nenachádza samotný kód, iba hlavičky metód s dokumentáciou
 * Hovoríme, že trieda implementuje rozhranie ak implementuje všetky jeho metódy
 */
public class Demo {
    public static void main(String[] args) {

        // do premennej typu rozhranie môžeme priradiť akýkoľvek objekt triedy, ktorá implementuje toto rozhranie
        Device phone = new Smartphone("Samsung", "Galaxy S9");

        Device cam = new Camera("MyCamera", "1");

        System.out.println("Mark of the phone is: " + phone.getMark()); // Samsung
        System.out.println("Mark of the cam is: " + cam.getMark()); // MyCamera

        // avsak po priradeni do premennej typu Device stráca niektoré metody, (resp presnejšie: prekladač ich nevidí),
        // napríklad getBatteryLevel()
        // Táto metóda sa nachádza v rozhraní Chargeable
        // phone.getBatteryLevel(); // nie je možné skompilovať
        // Objekt však túto metodu nestratil úplne po pretypovaní na typ Chargeable budeme mocť získať stav batérie
        Chargeable chargeablePhone = (Chargeable) phone;
        System.out.println("Battery level of the phone is: " + chargeablePhone.getBatteryLevel());

        // pretypovanie v jave má následovný tvar (<nový typ>) <objekt na pretypovanie>
        // pretypovanie nevytvára kópiu, ukazuje stále na ten istý objekt
        Smartphone samsungPhone = (Smartphone) chargeablePhone;
        System.out.println("Discharge 20% from phone battery!");
        samsungPhone.discharge(20); // battery level = 20
        // premenná samsungPhone a chargeablePhone ukazujú na ten istý objekt
        System.out.println("Battery level of the phone is: " + chargeablePhone.getBatteryLevel());
        System.out.println("Battery level of the phone is: " + samsungPhone.getBatteryLevel());

        Camera camera = new Camera("Nikon", "D5600");
        // Premenná typu rozhranie je dobré použiť ak chceme pracovať s viacerými triedami ale potrebujeme iba vlastnosti z
        // určitého rozhrania, ktoré tieto triedy implementujú
        // zaujíma nás len značka a model zariadenia preto metóda getInfoAboutDevice berie parameter typu Device
        Informer.printInfoAboutDevice(samsungPhone);
        Informer.printInfoAboutDevice(camera);

    }
}
