package cz.muni.fi.pb162.interfaces.devices;

public class Camera implements Device, Chargeable, BatteryReplaceable {

    private String mark;
    private String model;
    private int batteryLevel;

    public Camera(String mark, String model) {
        this.mark = mark;
        this.model = model;
        this.batteryLevel = 40;
    }

    // anotácia override určuje, že metóda je z rozhrania
    @Override
    public int getBatteryLevel() {
        return batteryLevel;
    }

    @Override
    public String getMark() {
        return mark;
    }

    @Override
    public String getModel() {
        return model;
    }

    @Override
    public void replaceBattery() {
        batteryLevel = 100;
    }

    // metóda charge sa nachádza v dvoch rozhraniach (BatteryReplaceable, Chargeable)
    // na rovnako pomenované metódy v rozhraniach platia rovnaké pravidlá ako pri preťažovaní metód
    // ak by mala metóda v rozhraniach rôzne návratove typy a inak sa nelíšila tak by nebolo možné aby trieda
    // implementovala obe rozhrania
    @Override
    public void charge(int percent) {
        if (!validate(percent, "charged")) {
            return;
        }
        if (percent + batteryLevel > 100) {
            batteryLevel = 100;
            return;
        }
        batteryLevel += percent;
    }

    @Override
    public void discharge(int percent) {
        if (!validate(percent, "discharged")) {
            return;
        }
        if (batteryLevel - percent < 0) {
            batteryLevel = 0;
            return;
        }
        batteryLevel -= percent;
    }

    private boolean validate(int percent, String message) {
        if (percent < 0 || percent > 100) {
            System.err.println("Device cannot be " + message + " negative or too high percent of battery life!");
            return false;
        }
        return true;
    }
}
